package com.ruoyi.zdsz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.annotation.DynamicName;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.helper.LoginHelper;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ruoyi.zdsz.domain.ZEngiineeringPhoto;
import com.ruoyi.zdsz.domain.ZEngineeringInfo;
import com.ruoyi.zdsz.domain.ZEngineeringNode;
import com.ruoyi.zdsz.domain.bo.*;
import com.ruoyi.zdsz.domain.vo.ZEngiineeringPhotoVo;
import com.ruoyi.zdsz.domain.vo.ZEngineeringIndustryVo;
import com.ruoyi.zdsz.domain.vo.ZEngineeringPipeJackingVo;
import com.ruoyi.zdsz.service.IZEngiineeringPhotoService;
import com.ruoyi.zdsz.service.IZEngineeringNodeService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.zdsz.domain.vo.ZEngineeringDangerousVo;
import com.ruoyi.zdsz.domain.ZEngineeringDangerous;
import com.ruoyi.zdsz.mapper.ZEngineeringDangerousMapper;
import com.ruoyi.zdsz.service.IZEngineeringDangerousService;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 危险工程Service业务层处理
 *
 * @author ruoyi
 * @date 2024-01-02
 */
@RequiredArgsConstructor
@Service
public class ZEngineeringDangerousServiceImpl implements IZEngineeringDangerousService {

    private final ZEngineeringDangerousMapper baseMapper;

    @Autowired
    private IZEngineeringNodeService izEngineeringNodeService;

    @Autowired
    private IZEngiineeringPhotoService photoService;

    /**
     * 查询危险工程列表
     */
    @Override
    public TableDataInfo<ZEngineeringDangerousVo> queryPageList(ZEngineeringDangerousBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZEngineeringDangerous> lqw = buildQueryWrapper(bo);
        Page<ZEngineeringDangerousVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询危险工程详情
     */
    @Override
    public ZEngineeringDangerousVo query(String id) {
        ZEngineeringDangerousVo vo = baseMapper.selectVoById(id);
        ZEngiineeringPhotoBo bo = new ZEngiineeringPhotoBo();
        bo.setParentId(vo.getId());
        bo.setCreateTime(vo.getCreateTime());
        vo.setPics(photoService.queryList(bo));
        ZEngineeringNodeBo zEngineeringNodeBo = new ZEngineeringNodeBo();
        zEngineeringNodeBo.setCivliId(vo.getId());
        zEngineeringNodeBo.setCreateTime(vo.getCreateTime());
        List<ZEngineeringNodeBo> zEngineeringNodeBos = izEngineeringNodeService.queryListDetails(zEngineeringNodeBo);
        vo.setzEngineeringNodeBoList(zEngineeringNodeBos);
        return vo;
    }

    /**
     * 查询危险工程列表
     */
    @Override
    public List<ZEngineeringDangerousVo> queryList(ZEngineeringDangerousBo bo) {
        LambdaQueryWrapper<ZEngineeringDangerous> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZEngineeringDangerous> buildQueryWrapper(ZEngineeringDangerousBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZEngineeringDangerous> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getBasicInformation()), ZEngineeringDangerous::getBasicInformation, bo.getBasicInformation());
        lqw.like(StringUtils.isNotBlank(bo.getEnginName()), ZEngineeringDangerous::getEnginName, bo.getEnginName());
        lqw.like(StringUtils.isNotBlank(bo.getEnginAddre()), ZEngineeringDangerous::getEnginAddre, bo.getEnginAddre());
        return lqw;
    }


    /**
     * 新增危险工程
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @DynamicName(spel = "#bo.createTime")
    public Boolean insert(ZEngineeringDangerousBo bo) {
        ZEngineeringDangerous add = BeanUtil.toBean(bo, ZEngineeringDangerous.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        bo.getzEngineeringNodeBo().setCivliId(add.getId());
        bo.getzEngineeringNodeBo().setCreateTime(add.getCreateTime());
        ZEngineeringNode entity = izEngineeringNodeService.insert(bo.getzEngineeringNodeBo());
        if (!ObjectUtil.isEmpty(entity)) {
            List<ZEngiineeringPhoto> list = new ArrayList<>();
            bo.getPics().stream().forEach(o -> {
                ZEngiineeringPhoto pic = new ZEngiineeringPhoto();
                pic.setParentId(bo.getId());
                pic.setPicUrl(o.getPicUrl());
                pic.setUpdateBy(String.valueOf(LoginHelper.getUserId()));
                pic.setUpdateTime(new Date());
                pic.setCreateTime(bo.getCreateTime());
                list.add(pic);
            });
            if (!list.isEmpty()) {
                photoService.insertBatch(list);
            }
        }
        return flag;
    }


    /**
     * 修改危险工程
     */
    @Override
    @DynamicName(spel = "#bo.createTime")
    @Transactional(rollbackFor = Exception.class)
    public Boolean update(ZEngineeringDangerousBo bo) {
        ZEngineeringDangerous update = BeanUtil.toBean(bo, ZEngineeringDangerous.class);
        validEntityBeforeSave(update);
        ZEngineeringNodeBo zEngineeringNodeBo = bo.getzEngineeringNodeBo();
        zEngineeringNodeBo.setCivliId(update.getId());
        zEngineeringNodeBo.setId(baseMapper.selectIdByCid(zEngineeringNodeBo.getCivliId()));
        zEngineeringNodeBo.setCreateTime(update.getCreateTime());
        izEngineeringNodeService.update(zEngineeringNodeBo);
        int i = baseMapper.updateById(update);
        if (i > 0) {
            List<ZEngiineeringPhoto> list = new ArrayList<>();

            // 查询数据库中已存在的文件列表
            ZEngiineeringPhotoBo bo1 = new ZEngiineeringPhotoBo();
            bo1.setParentId(update.getId());
            List<String> existingPhotos = photoService.queryList(bo1).stream()
                .map(ZEngiineeringPhotoVo::getPicUrl)
                .collect(Collectors.toList());

            // 插入前端传来的新文件
            for (ZEngiineeringPhotoVo pic : bo.getPics()) {
                if (!existingPhotos.contains(pic.getPicUrl())) {
                    ZEngiineeringPhoto newPhoto = new ZEngiineeringPhoto();
                    newPhoto.setParentId(bo.getId());
                    newPhoto.setPicUrl(pic.getPicUrl());
                    newPhoto.setFileName(pic.getFileName());
                    newPhoto.setCreateBy(String.valueOf(LoginHelper.getUserId()));
                    newPhoto.setCreateTime(new Date());
                    list.add(newPhoto);
                }
            }

            // 删除数据库中多余的文件
            List<String> photosToDelete = existingPhotos.stream()
                .filter(pic -> !bo.getPics().stream().map(ZEngiineeringPhotoVo::getPicUrl).collect(Collectors.toList()).contains(pic))
                .collect(Collectors.toList());
            if (!photosToDelete.isEmpty()) {
                photosToDelete.forEach(item -> {
                    photoService.deleteWithValidByurls(item);
                });
            }

            // 批量插入新图片
            if (!list.isEmpty()) {
                photoService.insertBatch(list);
            }
        }
        return i > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZEngineeringDangerous entity) {
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除危险工程
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<String> ids, Boolean isValid) {
        if (isValid) {
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        ids.forEach(item -> {
            ZEngineeringDangerousVo vo = baseMapper.selectVoById(item);
            ZEngineeringNodeBo zEngineeringNodeBo = new ZEngineeringNodeBo();
            zEngineeringNodeBo.setCreateTime(vo.getCreateTime());
            zEngineeringNodeBo.setCivliId(vo.getId());
            List<ZEngineeringNodeBo> zEngineeringNodeBos = izEngineeringNodeService.queryListDetails(zEngineeringNodeBo);
            zEngineeringNodeBos.stream().forEach(ite ->
                izEngineeringNodeService.delete(ite)
            );
        });
        return baseMapper.deleteBatchIds(ids) > 0;
    }

    /**
     * 危险工程历史
     */
    @Override
    public ZEngineeringDangerousVo queryEngineeringDangerous(String id) {
        ZEngineeringDangerousVo vo = baseMapper.selectVoById(id);
        ZEngineeringNodeBo zEngineeringNodeBo = new ZEngineeringNodeBo();
        zEngineeringNodeBo.setCreateTime(vo.getCreateTime());
        zEngineeringNodeBo.setCivliId(vo.getId());
        vo.setzEngineeringNodeBoList(izEngineeringNodeService.queryListDetails(zEngineeringNodeBo));
        return vo;
    }


    /**
     * 新增危险工程（无节点）
     */
    @Override
    public Boolean insertByBo(ZEngineeringDangerousBo bo) {
        ZEngineeringDangerous add = BeanUtil.toBean(bo, ZEngineeringDangerous.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        if (!bo.getPics().isEmpty()) {
            List<ZEngiineeringPhoto> list = new ArrayList<>();
            bo.getPics().stream().forEach(o -> {
                ZEngiineeringPhoto pic = new ZEngiineeringPhoto();
                pic.setParentId(bo.getId());
                pic.setPicUrl(o.getPicUrl());
                pic.setFileName(o.getFileName());
                pic.setCreateBy(String.valueOf(LoginHelper.getUserId()));
                pic.setCreateTime(new Date());
                pic.setUpdateBy(String.valueOf(LoginHelper.getUserId()));
                pic.setUpdateTime(new Date());
                list.add(pic);
            });
            if (!list.isEmpty()) {
                photoService.insertBatch(list);
            }
        }
        return flag;
    }

    /**
     * 修改危险工程（无节点）
     */
    @Override
    public Boolean updateByBo(ZEngineeringDangerousBo bo) {
        ZEngineeringDangerous update = BeanUtil.toBean(bo, ZEngineeringDangerous.class);
        validEntityBeforeSave(update);
        int i = baseMapper.updateById(update);
        if (i > 0) {
            List<ZEngiineeringPhoto> list = new ArrayList<>();

            // 查询数据库中已存在的文件列表
            ZEngiineeringPhotoBo bo1 = new ZEngiineeringPhotoBo();
            bo1.setParentId(update.getId());
            List<String> existingPhotos = photoService.queryList(bo1).stream()
                .map(ZEngiineeringPhotoVo::getPicUrl)
                .collect(Collectors.toList());

            // 插入前端传来的新文件
            for (ZEngiineeringPhotoVo pic : bo.getPics()) {
                if (!existingPhotos.contains(pic.getPicUrl())) {
                    ZEngiineeringPhoto newPhoto = new ZEngiineeringPhoto();
                    newPhoto.setParentId(bo.getId());
                    newPhoto.setPicUrl(pic.getPicUrl());
                    newPhoto.setFileName(pic.getFileName());
                    newPhoto.setCreateBy(String.valueOf(LoginHelper.getUserId()));
                    newPhoto.setCreateTime(new Date());
                    list.add(newPhoto);
                }
            }

            // 删除数据库中多余的文件
            List<String> photosToDelete = existingPhotos.stream()
                .filter(pic -> !bo.getPics().stream().map(ZEngiineeringPhotoVo::getPicUrl).collect(Collectors.toList()).contains(pic))
                .collect(Collectors.toList());
            if (!photosToDelete.isEmpty()) {
                photosToDelete.forEach(item -> {
                    photoService.deleteWithValidByurls(item);
                });
            }

            // 批量插入新图片
            if (!list.isEmpty()) {
                photoService.insertBatch(list);
            }
        }
        return i > 0;
    }

    /**
     * 查询危险工程（无节点）
     */
    @Override
    public ZEngineeringDangerousVo queryById(String id) {
        ZEngineeringDangerousVo vo = baseMapper.selectVoById(id);
        ZEngiineeringPhotoBo bo = new ZEngiineeringPhotoBo();
        bo.setParentId(vo.getId());
        bo.setCreateTime(vo.getCreateTime());
        vo.setPics(photoService.queryList(bo));
        return vo;
    }
    @Override
    public List<Map<String, Object>> selectEnginNameAndId(ZEngineeringDangerousBo bo) {
        LambdaQueryWrapper<ZEngineeringDangerous> lqw = buildQueryWrapper(bo);
        List<ZEngineeringDangerousVo> zEngineeringDangerousVos = baseMapper.selectVoList(lqw);
        List<Map<String, Object>> filteredList = zEngineeringDangerousVos.stream()
            .map(vo -> {
                Map<String, Object> map = new HashMap<>();
                map.put("name", vo.getEnginName());
                map.put("id", vo.getId());
                return map;
            })
            .collect(Collectors.toList());
        return filteredList;
    }
}
